summaryrefslogtreecommitdiff
path: root/src/runtime/lfstack_linux_power64x.go
diff options
context:
space:
mode:
authorRuss Cox <rsc@golang.org>2014-11-24 12:07:11 -0500
committerRuss Cox <rsc@golang.org>2014-11-24 12:07:11 -0500
commite5fc9ffb729e31c4eb0a6518e819e9fc70f14818 (patch)
tree5f67f9fb6c209085505b8fa6a01a245942ea45c3 /src/runtime/lfstack_linux_power64x.go
parentdf7d4576f312e1b646af4f5f1a32412a4221b785 (diff)
parentdb6a11ff558add790779fdef118b5ad899f77137 (diff)
downloadgo-e5fc9ffb729e31c4eb0a6518e819e9fc70f14818.tar.gz
[dev.garbage] all: merge dev.cc (493ad916c3b1) into dev.garbage
TBR=austin CC=golang-codereviews https://codereview.appspot.com/179290043
Diffstat (limited to 'src/runtime/lfstack_linux_power64x.go')
-rw-r--r--src/runtime/lfstack_linux_power64x.go20
1 files changed, 13 insertions, 7 deletions
diff --git a/src/runtime/lfstack_linux_power64x.go b/src/runtime/lfstack_linux_power64x.go
index 7a122bf92..89e389fc7 100644
--- a/src/runtime/lfstack_linux_power64x.go
+++ b/src/runtime/lfstack_linux_power64x.go
@@ -9,18 +9,24 @@ package runtime
import "unsafe"
// On Power64, Linux limits the user address space to 43 bits.
// (https://www.kernel.org/doc/ols/2001/ppc64.pdf)
// In addition to the 21 bits taken from the top, we can take 3 from the
// bottom, because node must be pointer-aligned, giving a total of 24 bits
// of count.
+const (
+ addrBits = 48
+ cntBits = 64 - addrBits + 3
+)
func lfstackPack(node *lfnode, cnt uintptr) uint64 {
- return uint64(uintptr(unsafe.Pointer(node)))<<21 | uint64(cnt&(1<<24-1))
+ return uint64(uintptr(unsafe.Pointer(node)))<<(64-addrBits) | uint64(cnt&(1<<cntBits-1))
}
func lfstackUnpack(val uint64) (node *lfnode, cnt uintptr) {
- node = (*lfnode)(unsafe.Pointer(uintptr(val >> 24 << 3)))
- cnt = uintptr(val & (1<<24 - 1))
+ node = (*lfnode)(unsafe.Pointer(uintptr(val >> cntBits << 3)))
+ cnt = uintptr(val & (1<<cntBits - 1))
return
}
@@ -9,18 +9,24 @@ package runtime
import "unsafe"
-// On Power64, Linux limits the user address space to 43 bits.
-// (https://www.kernel.org/doc/ols/2001/ppc64.pdf)
-// In addition to the 21 bits taken from the top, we can take 3 from the
-// bottom, because node must be pointer-aligned, giving a total of 24 bits
+// On Power64, Linux limits the user address space to 46 bits (see
+// TASK_SIZE_USER64 in the Linux kernel). This has grown over time,
+// so here we allow 48 bit addresses.
+//
+// In addition to the 16 bits taken from the top, we can take 3 from the
+// bottom, because node must be pointer-aligned, giving a total of 19 bits
// of count.